# -*- coding: utf8 -*-
from __future__ import unicode_literals

import time
import logging
from unidecode import unidecode

from driverng.astm import (
    AstmMessageV2,
    Segment,
    HeaderSegment,
    HprimList as HL7List,
    HprimString as HL7String,
    HprimDate as HL7Date,
    HprimForeign as HL7Foreign,
)

logger = logging.getLogger("driverng.infectio")


class InfectioSegmentMSH(HeaderSegment):
    sending_application = HL7String(3, default="MCA")
    sending_facility = HL7String(4, default="MCA")
    receiving_application = HL7String(5, default="P4L")
    receiving_facility = HL7String(6, default="P4L")
    message_date = HL7Date(7)
    security = HL7String(8)
    message_type = HL7String((9, 1), default="OUL")
    message_code = HL7String((9, 2), default="R22")
    control_id = HL7String(10)
    processing_id = HL7String(11, default="P")
    version_id = HL7String(12, default="2.5")
    sequence_number = HL7String(13)
    accept_acknowledgment_type = HL7String(15, default="NE")
    application_acknowledgment = HL7String(16, default="AL")
    character_set = HL7String(18, default="UNICODE UTF-8")

    patients = HL7Foreign("PID", "header")
    orders = HL7Foreign("OBR", "header")
    errors = HL7Foreign("ERR", "header")


class InfectioSegmentPID(Segment):
    patient_id = HL7String(4)
    alternate_patient_id = HL7String(5)
    lastname = HL7String((6, 1))
    firstname = HL7String((6, 2))
    mother_maiden_name = HL7String(7)
    birthdate = HL7Date(8, fmt_date="%Y%m%d")
    sex = HL7String(9)
    adress = HL7String((14, 1))
    postal_code = HL7String((14, 2))
    city = HL7String((14, 3))

    demographics_adding = HL7Foreign("PD1", "patient")
    visits = HL7Foreign("PV1", "patient")
    orders = HL7Foreign("OBR", "patient")
    infos = HL7Foreign("NTE", "patient")


class InfectioSegmentPD1(Segment):
    # Patient Primary Care Provider Name
    physician_code = HL7String((5, 1))
    physician_lastname = HL7String((5, 2))
    physician_firstname = HL7String((5, 3))
    physician_title = HL7String((5, 7))

    infos = HL7Foreign("NTE", "patient")
    visits = HL7Foreign("PV1", "patient")


class InfectioSegmentPV1(Segment):
    patient_class = HL7String(3)
    location = HL7String((4, 2))
    attending_doctor_code = HL7String((8, 1))
    attending_doctor_lastname = HL7String((8, 2))
    attending_doctor_firstname = HL7String((8, 3))

    orders = HL7Foreign("OBR", "patient")


class InfectioSegmentNTE(Segment):
    comment_source = HL7String(3)
    comment_text = HL7String(4)
    comment_type1 = HL7String((5, 1))
    comment_type2 = HL7String((5, 2))
    comment_type3 = HL7String((5, 3))


class InfectioSegmentOBR(Segment):
    placer_order_number = HL7String(3)
    universal_service_id_code = HL7String((5, 1))
    universal_service_id_label = HL7String((5, 2))
    priority = HL7String(6)
    request_date = HL7Date(7)
    observation_date = HL7Date(8)
    collection_volume = HL7String(10)
    action_code = HL7String(12)
    specimen_source = HL7String((15, 1))
    specimen_source_rank = HL7String((15, 2))
    received_date = HL7Date(16)
    ordering_provider = HL7String(17)
    diagnostic = HL7String(25)

    results = HL7Foreign("OBX", "order")
    common_order = HL7Foreign("ORC", "order")
    sample_priority = HL7Foreign("TQ1", "order")


class InfectioSegmentOBX(Segment):
    value_type = HL7String(3)
    assay_number = HL7String((4, 1))
    result_type_code = HL7String((4, 2))
    observation_sub_id = HL7String(5)
    value = HL7String((6, 1))
    value_label = HL7String((6, 2))
    units = HL7String((7, 1))
    units_label = HL7String((7, 2))
    units_coding = HL7String((7, 3))
    reference_range = HL7String(8)
    # Pour les ATB, il s'agit du résultat expertisé S ou I ou R ou X
    interpretation_codes = HL7List(9)
    # Statut du résultat
    # F: Final
    # P: Provisoire
    # Transmission du résultat validé final
    observ_result_status = HL7String(12, default="F")
    date_observation = HL7Date(15)
    responsible_observer = HL7String(17)
    equipement_identifier = HL7List(19)
    date_analysis = HL7Date(20)
    observation_type = HL7String(30)

    orders = HL7Foreign("OBR", "patient")
    infos = HL7Foreign("NTE", "result")


class InfectioSegmentERR(Segment):
    error_location = HL7String(3)
    hl7_error_code = HL7String(4)
    severity = HL7String(5)
    application_error_code = HL7String(6)
    application_error_parameter = HL7String(7)


class InfectioSegmentORC(Segment):
    order_control = HL7String(2)
    placer_order_number = HL7String(3)
    filler_group_number = HL7String(4)
    order_status = HL7String(6)
    response_flag = HL7String(7)
    quantity = HL7String(8)
    parent = HL7String(9)
    date_transaction = HL7Date(10)
    entered_by = HL7String(11)
    verified_by = HL7String(12)

    sample_priority = HL7Foreign("TQ1", "patient")
    results = HL7Foreign("OBX", "order")

    def to_string(self, separators=None):
        self.rang_segment = ""
        return super(InfectioSegmentORC, self).to_string(separators=separators)


class InfectioSegmentTQ1(Segment):
    start_datetime = HL7String(8)
    # Request Urgency
    priority_code = HL7String(10)

    orders = HL7Foreign("OBR", "patient")
    results = HL7Foreign("OBX", "order")


class Driver(AstmMessageV2):
    SPEC = {
        "code": "infectio",
        "label": "Infectio Epidemio"
    }

    SEGMENT_CLASS_MAP = {
        "MSH": InfectioSegmentMSH,
        "ERR": InfectioSegmentERR,
        "PID": InfectioSegmentPID,
        "PD1": InfectioSegmentPD1,
        "PV1": InfectioSegmentPV1,
        "NTE": InfectioSegmentNTE,
        "ORC": InfectioSegmentORC,
        "OBR": InfectioSegmentOBR,
        "OBX": InfectioSegmentOBX,
        "TQ1": InfectioSegmentTQ1,
    }
    DEFAULT_SEPARATORS = "|^~\\&"
    FIRST_SEP_INDEX = 3
    SEGMENT_ENDING = "\r"

    OPTIONS = [
        AstmMessageV2.Option(
            "infectio_sending_application",
            label="Sending Application MSH.3",
            default=""
        ),
        AstmMessageV2.Option(
            "infectio_sending_facility",
            label="Sending Facility MSH.4",
            default="INFECTIO"
        ),
        AstmMessageV2.Option(
            "infectio_receiving_application",
            label="Receiving Application MSH.5",
            default="",
        ),
        AstmMessageV2.Option(
            "infectio_receiving_facility",
            label="Receiving Facility MSH.6",
            default="INFECTIO",
        ),
    ]

    def __init__(
        self,
        infectio_sending_application="",
        infectio_sending_facility="INFECTIO",
        infectio_receiving_application="",
        infectio_receiving_facility="INFECTIO",
        **kwargs
    ):
        self.sending_application = infectio_sending_application
        self.sending_facility = infectio_sending_facility
        self.receiving_application = infectio_receiving_application
        self.receiving_facility = infectio_receiving_facility
        super(Driver, self).__init__(**kwargs)

    def create(self, **kwargs):
        return InfectioSegmentMSH(type_segment="MSH", **kwargs)

    def to_string(self, message, separators=None, **kwargs):
        return super(Driver, self).to_string(message, separators, **kwargs) + "\r"

    def encode_worklist(self, worklist, context):
        msg = self.create(
            sending_application=self.sending_application,
            sending_facility=self.sending_facility,
            receiving_application=self.receiving_application,
            receiving_facility=self.receiving_facility,
            message_date=context["now"],
            control_id=context.get(
                "control_id", int(time.time() * 1000))
        )
        for patient in worklist.get_astm_worklist(
            unique_together=["action", "priority"],
            priority={"A": "S", "S": "S", worklist.ELSE: "R"},
        ):
            seg_pid = msg.patients.add(
                patient_id=patient.patient_id,
                lastname=unidecode(patient.last_name),
                firstname=unidecode(patient.first_name),
                birthdate=patient.birthdate,
                sex=patient.sex,
                adress=patient.address,
                city=patient.city,
                postal_code=patient.postal_code,
            )
            physician = [p for p in patient.physicians if p.type.lower() == 'attending']
            prescriber = [p for p in patient.physicians if p.type.lower() == 'prescriber']
            if physician:
                seg_pd1 = seg_pid.demographics_adding.add(
                    physician_code=physician[0].code,
                    physician_lastname=physician[0].last_name,
                    physician_firstname=physician[0].first_name,
                    physician_title=physician[0].title
                )
            else:
                seg_pd1 = seg_pid
            if patient.location or prescriber:
                seg_pv1 = seg_pd1.visits.add(
                    location=patient.location,
                    attending_doctor_code=prescriber[0].code if prescriber else "",
                    attending_doctor_lastname=prescriber[0].last_name if prescriber else "",
                    attending_doctor_firstname=prescriber[0].first_name if prescriber else ""
                )
            else:
                seg_pv1 = seg_pd1
            result_to_send = False
            for sample, action, priority in patient.samples:
                for result in sample.results:
                    result_to_send = True
                    # Encodage de l'identification du germe présent sous forme
                    # de résultat json dans le résultat
                    if result.get("resu_json") and "germes" in result["resu_json"]:
                        for resu_json_germe in result["resu_json"]["germes"]:
                            if not isinstance(resu_json_germe, dict):
                                continue
                            seg_obr = seg_pv1.orders.add(
                                placer_order_number=sample.sample_id,
                                universal_service_id_code="Identification",
                                universal_service_id_label="Identification",
                                request_date=sample.request_date,
                                specimen_source=sample.specimen,
                                received_date=result.date,
                            )
                            rank_germ = resu_json_germe.get("rang", "")
                            keys_germe = [cle for cle in resu_json_germe.keys() if cle.lower() == 'id']
                            id_germ = resu_json_germe.get(keys_germe[0] if keys_germe else None)
                            if isinstance(id_germ, list):
                                id_germ = ""
                            seg_obr.results.add(
                                value_type="CE",
                                assay_number="CultAerobie",
                                result_type_code="Culture aerobie",
                                # Rang du germe
                                # Mettre le rang du germe à 1 si pas de
                                # rang du germe notamment lorsqu'un germe a
                                # été créé automatiquement par une règle
                                observation_sub_id=str(rank_germ) if rank_germ else "1",
                                value=id_germ[2:] if id_germ.startswith("G_") else id_germ,
                                value_label=id_germ[2:] if id_germ.startswith("G_") else id_germ,
                                interpretation_codes=result.flags,
                            )
                    elif result.get("atb"):
                        # Segment OBR avec le germe de l'ATB
                        seg_obr = seg_pv1.orders.add(
                            placer_order_number=sample.sample_id,
                            universal_service_id_code=result.code,
                            universal_service_id_label=result.code,
                            request_date=sample.request_date,
                            specimen_source=sample.specimen,
                            received_date=result.date,
                        )
                        seg_obr.results.add(
                            value_type="CE",
                            assay_number="CultAerobie",
                            result_type_code="Culture aerobie",
                            # Rang du germe?
                            observation_sub_id=result.sub_code,
                            value=result.value[2:] if result.value.startswith("G_") else result.value,
                            value_label=result.value_label,
                            interpretation_codes=result.flags,
                        )
                        if result.get("phenotypes"):
                            # Encodage des phenotypes
                            for phenotype in result.phenotypes:
                                seg_obr.results.add(
                                    value_type="ST",
                                    assay_number="IDENT_PHENO{}".format(result.code),
                                    observation_sub_id=result.sub_code,
                                    value=phenotype,
                                )
                        # Segment OBR précédent la batterie des
                        # antibiotiques
                        seg_obr = seg_pv1.orders.add(
                            placer_order_number=sample.sample_id,
                            universal_service_id_code="Antibiogramme",
                            universal_service_id_label="Antibiogramme",
                            request_date=sample.request_date,
                            specimen_source=sample.specimen,
                            specimen_source_rank=result.sub_code,
                            received_date=result.date,
                        )
                        for antibio in result.atb:
                            seg_obr.results.add(
                                value_type="SN",
                                assay_number=antibio.get("ANTIBIO").split("_", 1)[1] if "_" in antibio.get("ANTIBIO", "") else antibio.get("ANTIBIO", ""),
                                result_type_code=antibio.get("LABEL_ANTIBIO"),
                                value=antibio.get("CMI"),
                                interpretation_codes=antibio.get("SENSI"),
                                date_observation=result.date,
                                responsible_observer=result.validator,
                            )
                    # Encodage de l'identification du germe présent sous forme
                    else:
                        seg_obr = seg_pv1.orders.add(
                            placer_order_number=sample.sample_id,
                            universal_service_id_code=result.code,
                            universal_service_id_label=result.label,
                            request_date=sample.request_date,
                            specimen_source=sample.specimen,
                            received_date=result.date,
                        )
                        seg_obr.results.add(
                            value_type=result.type,
                            assay_number=result.code,
                            result_type_code=result.label,
                            value=result.value,
                            value_label=result.value_label or result.value,
                            interpretation_codes=result.flags,
                        )

        if len(msg.patients) and result_to_send:
            return msg
